@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ import socket
 from cStringIO import StringIO
 
 from .. import (log, LOG_CHECK, gzip2 as gzip, strformat, url as urlutil,
-    httplib2 as httplib, LinkCheckerError, configuration, get_link_pat)
+    httplib2 as httplib, LinkCheckerError, get_link_pat, httputil)
 from . import (internpaturl, proxysupport, httpheaders as headers, urlbase,
     get_url_from)
 # import warnings
@@ -37,6 +37,9 @@ from .const import WARN_HTTP_ROBOTS_DENIED, \
     WARN_HTTP_DECOMPRESS_ERROR, WARN_HTTP_UNSUPPORTED_ENCODING, \
     WARN_HTTP_AUTH_UNKNOWN
 
+# assumed HTTP header encoding
+HEADER_ENCODING = "iso-8859-1"
+
 # helper alias
 unicode_safe = strformat.unicode_safe
 
@@ -102,7 +105,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             # Note that content-encoding causes size differences since
             # the content data is always decoded.
             try:
-                self.size = int(self.headers["Content-Length"])
+                self.size = int(self.getheader("Content-Length"))
                 if self.dlsize == -1:
                     self.dlsize = self.size
             except (ValueError, OverflowError):
@@ -147,7 +150,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # check the http connection
         response = self.check_http_connection()
         if self.headers and "Server" in self.headers:
-            server = self.headers['Server']
+            server = self.getheader('Server')
         else:
             server = _("unknown")
         if self.fallback_get:
@@ -197,11 +200,12 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 response.reason = unicode_safe(response.reason)
             log.debug(LOG_CHECK,
                 "Response: %s %s", response.status, response.reason)
-            log.debug(LOG_CHECK, "Headers: %s", self.headers)
+            uheaders = unicode_safe(self.headers, encoding=HEADER_ENCODING)
+            log.debug(LOG_CHECK, "Headers: %s", uheaders)
             # proxy enforcement (overrides standard proxy)
             if response.status == 305 and self.headers:
                 oldproxy = (self.proxy, self.proxyauth)
-                newproxy = self.headers.getheader("Location")
+                newproxy = self.getheader("Location")
                 self.add_info(_("Enforced proxy `%(name)s'.") %
                               {"name": newproxy})
                 self.set_proxy(newproxy)
@@ -243,7 +247,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 return response
             # user authentication
             if response.status == 401:
-                authenticate = self.headers.get('WWW-Authenticate')
+                authenticate = self.getheader('WWW-Authenticate')
                 if not authenticate or not authenticate.startswith("Basic"):
                     # LinkChecker only supports Basic authorization
                     args = {"auth": authenticate}
@@ -253,24 +257,29 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                        tag=WARN_HTTP_AUTH_UNKNOWN)
                     return
                 if not self.auth:
-                    import base64
                     _user, _password = self.get_user_password()
                     self.auth = "Basic " + \
-                        base64.encodestring("%s:%s" % (_user, _password))
+                        httputil.encode_base64("%s:%s" % (_user, _password))
                     log.debug(LOG_CHECK,
                         "Authentication %s/%s", _user, _password)
                     continue
             elif response.status >= 400:
-                # retry with GET (but do not set fallback flag)
+                # Retry with GET, but do not set fallback_get flag.
+                # This ensures that sites that do not properly support HEAD
+                # requests do not give false errors.
                 if self.method == "HEAD" and self.method_get_allowed:
-                    self.method = "GET"
-                    self.aliases = []
-                    continue
+                    # The squid proxy reports valid 200 instead of 404 due to
+                    # garbage sent from the server at the start of the GET
+                    # request. See http://www.aldec.com/Products
+                    if u'squid' not in self.getheader('Via', u'').lower():
+                        self.method = "GET"
+                        self.aliases = []
+                        continue
             elif self.headers and self.method == "HEAD" and self.method_get_allowed:
                 # test for HEAD support
                 mime = self.get_content_type()
-                poweredby = self.headers.get('X-Powered-By', '')
-                server = self.headers.get('Server', '')
+                poweredby = self.getheader('X-Powered-By', u'')
+                server = self.getheader('Server', u'')
                 if ((mime in ('application/octet-stream', 'text/plain') and
                   (poweredby.startswith('Zope') or server.startswith('Zope')))
                  or ('ASP.NET' in poweredby and 'Microsoft-IIS' in server)):
@@ -308,8 +317,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
 
     def follow_redirection (self, response, set_result, redirected):
         """Follow one redirection of http response."""
-        newurl = self.headers.getheader("Location",
-                     self.headers.getheader("Uri", ""))
+        newurl = self.getheader("Location",
+                     self.getheader("Uri", u""))
         # make new url absolute and unicode
         newurl = urlparse.urljoin(redirected, unicode_safe(newurl))
         log.debug(LOG_CHECK, "Redirected to %r", newurl)
@@ -339,6 +348,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # check cache again on the changed URL
         if self.aggregate.urlqueue.checked_redirect(redirected, self):
             return -1, response
+        # store cookies from redirect response
+        self.store_cookies()
         # new response data
         response.close()
         response = self._try_http_response()
@@ -360,17 +371,15 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
 
     def check_redirection_domain (self, redirected, urlparts, set_result, response):
         """Return True if redirection domain is ok, else False."""
-        if urlparts[1] == self.urlparts[1]:
-            # URL domain did not change
-            return True
-        # URL domain changed
-        if self.recursion_level == 0 and urlparts[0] in ('http', 'https'):
-            # Add intern patterns for redirection of URLs given by the
-            # user for HTTP schemes.
-            pat = internpaturl.get_intern_pattern(redirected)
-            log.debug(LOG_CHECK, "Add intern pattern %r", pat)
-            self.aggregate.config['internlinks'].append(get_link_pat(pat))
-            return True
+        if urlparts[1] != self.urlparts[1]:
+            # URL domain changed
+            if self.recursion_level == 0 and urlparts[0] in ('http', 'https'):
+                # Add intern patterns for redirection of URLs given by the
+                # user for HTTP schemes.
+                pat = internpaturl.get_intern_pattern(redirected)
+                log.debug(LOG_CHECK, "Add intern pattern %r", pat)
+                self.aggregate.config['internlinks'].append(get_link_pat(pat))
+                return True
         # check extern filter again
         self.set_extern(redirected)
         if self.extern[0] and self.extern[1]:
@@ -441,6 +450,17 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                              tag=WARN_HTTP_MOVED_PERMANENT)
             self.has301status = True
 
+    def getheader (self, name, default=None):
+        """Get decoded header value.
+
+        @return: decoded header value or default of not found
+        @rtype: unicode or type of default
+        """
+        value = self.headers.get(name)
+        if value is None:
+            return default
+        return unicode_safe(value, encoding=HEADER_ENCODING)
+
     def get_alias_cache_data (self):
         """
         Return all data values that should be put in the cache,
@@ -463,21 +483,12 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 self.add_warning(unicode_safe(response.reason),
                                  tag=WARN_HTTP_EMPTY_CONTENT)
             # store cookies for valid links
-            if self.aggregate.config['storecookies']:
-                for c in self.cookies:
-                    self.add_info(_("Sent cookie: %(cookie)s.") %
-                                  {"cookie": c})
-                jar = self.aggregate.cookies.add(self.headers, self.urlparts[0],
-                                                 self.urlparts[1], self.urlparts[2])
-                if not jar:
-                    self.add_warning(_("Could not store cookies from headers: %(headers)s.") %
-                                     {'headers': str(self.headers)},
-                                     tag=WARN_HTTP_COOKIE_STORE_ERROR)
+            self.store_cookies()
             if response.status >= 200:
                 self.set_result(u"%r %s" % (response.status, response.reason))
             else:
                 self.set_result(u"OK")
-        modified = self.headers.get('Last-Modified', '')
+        modified = self.getheader('Last-Modified', u'')
         if modified:
             self.add_info(_("Last modified %(date)s.") % {"date": modified})
 
@@ -542,24 +553,17 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if (self.parent_url and
             self.parent_url.startswith(('http://', 'https://'))):
             self.url_connection.putheader("Referer", self.parent_url)
-        self.url_connection.putheader("User-Agent", configuration.UserAgent)
+        self.url_connection.putheader("User-Agent",
+            self.aggregate.config["useragent"])
         self.url_connection.putheader("Accept-Encoding",
                                   "gzip;q=1.0, deflate;q=0.9, identity;q=0.5")
         if self.aggregate.config['sendcookies']:
-            scheme = self.urlparts[0]
-            host = self.urlparts[1]
-            port = urlutil.default_ports.get(scheme, 80)
-            host, port = urllib.splitnport(host, port)
-            path = self.urlparts[2]
-            self.cookies = self.aggregate.cookies.get(scheme, host, port, path)
-            for c in self.cookies:
-                name = c.client_header_name()
-                value = c.client_header_value()
-                self.url_connection.putheader(name, value)
+            self.send_cookies()
         self.url_connection.endheaders()
         response = self.url_connection.getresponse(True)
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
+        self.content_type = None
         self.persistent = not response.will_close
         if self.persistent and self.method == "HEAD":
             # Some servers send page content after a HEAD request,
@@ -571,7 +575,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # Note that for POST method the connection should also be closed,
         # but this method is never used.
         if self.persistent and (self.method == "GET" or
-           self.headers.getheader("Content-Length") != "0"):
+           self.getheader("Content-Length") != u"0"):
             # always read content from persistent connections
             self._read_content(response)
             assert not response.will_close
@@ -580,6 +584,53 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             response.reason = httplib.responses[response.status]
         return response
 
+    def store_cookies (self):
+        """Save cookies from response headers."""
+        if self.aggregate.config['storecookies']:
+            for c in self.cookies:
+                self.add_info(_("Sent Cookie: %(cookie)s.") %
+                              {"cookie": c.client_header_value()})
+            errors = self.aggregate.cookies.add(self.headers,
+                self.urlparts[0], self.urlparts[1], self.urlparts[2])
+            if errors:
+                self.add_warning(
+                  _("Could not store cookies from headers: %(error)s.") %
+                   {'error': "\n".join(errors)},
+                   tag=WARN_HTTP_COOKIE_STORE_ERROR)
+
+    def send_cookies (self):
+        """Add cookie headers to request."""
+        scheme = self.urlparts[0]
+        host = self.urlparts[1]
+        port = urlutil.default_ports.get(scheme, 80)
+        host, port = urllib.splitnport(host, port)
+        path = self.urlparts[2]
+        self.cookies = self.aggregate.cookies.get(scheme, host, port, path)
+        if not self.cookies:
+            return
+        # add one cookie header with all cookie data
+        # this is limited by maximum header length
+        headername = "Cookie"
+        headervalue = ""
+        max_value_len = headers.MAX_HEADER_BYTES - len(headername) - 2
+        for c in self.cookies:
+            cookievalue = c.client_header_value()
+            if "version" in c.attributes:
+                # add separate header for explicit versioned cookie
+                if headervalue:
+                    self.url_connection.putheader(headername, headervalue)
+                self.url_connection.putheader(headername, cookievalue)
+                headervalue = ""
+                continue
+            if headervalue:
+                cookievalue = "; " + cookievalue
+            if (len(headervalue) + len(cookievalue)) < max_value_len:
+                headervalue += cookievalue
+            else:
+                log.debug(LOG_CHECK, "Discard too-long cookie %r", cookievalue)
+        if headervalue:
+            self.url_connection.putheader(headername, headervalue)
+
     def get_http_object (self, host, scheme):
         """
         Open a HTTP connection.
@@ -623,6 +674,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         response = self._try_http_response()
         response = self.follow_redirections(response, set_result=False)[1]
         self.headers = response.msg
+        self.content_type = None
+        self.charset = headers.get_charset(self.headers)
         # Re-read size info, since the GET request result could be different
         # than a former HEAD request.
         self.add_size_info()
