@@ -139,6 +139,7 @@ class UrlBase (object):
         self.column = column
         self.name = name
         self.encoding = url_encoding
+        self.charset = None
         if self.base_ref:
             assert not urlutil.url_needs_quoting(self.base_ref), \
                    "unquoted base reference URL %r" % self.base_ref
@@ -235,24 +236,27 @@ class UrlBase (object):
 
     def set_title_from_content (self):
         """Set title of page the URL refers to.from page content."""
-        if self.valid:
-            try:
-                handler = linkparse.TitleFinder()
-            except tuple(ExcList):
-                return
-            parser = htmlsax.parser(handler)
-            handler.parser = parser
-            # parse
-            try:
-                parser.feed(self.get_content())
-                parser.flush()
-            except linkparse.StopParse, msg:
-                log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
-            # break cyclic dependencies
-            handler.parser = None
-            parser.handler = None
-            if handler.title:
-                self.title = handler.title
+        if not self.valid:
+            return
+        try:
+            handler = linkparse.TitleFinder()
+        except tuple(ExcList):
+            return
+        parser = htmlsax.parser(handler)
+        handler.parser = parser
+        if self.charset:
+            parser.encoding = self.charset
+        # parse
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
+        # break cyclic dependencies
+        handler.parser = None
+        parser.handler = None
+        if handler.title:
+            self.title = handler.title
 
     def is_parseable (self):
         """
@@ -301,6 +305,7 @@ class UrlBase (object):
         """
         Fill attributes from cache data.
         """
+        self.url = cache_data["url"]
         self.result = cache_data["result"]
         self.has_result = True
         anchor_changed = (self.anchor != cache_data["anchor"])
@@ -326,7 +331,8 @@ class UrlBase (object):
 
     def get_cache_data (self):
         """Return all data values that should be put in the cache."""
-        return {"result": self.result,
+        return {"url": self.url,
+                "result": self.result,
                 "warnings": self.warnings,
                 "info": self.info,
                 "valid": self.valid,
@@ -599,6 +605,8 @@ class UrlBase (object):
         handler = linkparse.MetaRobotsFinder()
         parser = htmlsax.parser(handler)
         handler.parser = parser
+        if self.charset:
+            parser.encoding = self.charset
         # parse
         try:
             parser.feed(self.get_content())
@@ -783,9 +791,7 @@ class UrlBase (object):
         """Check HTML syntax of this page (which is supposed to be HTML)
         with the local HTML tidy module."""
         if not fileutil.has_module("tidy"):
-            log.warn(LOG_CHECK, _("warning: tidy module is not available; " \
-                 "download from http://utidylib.berlios.de/"))
-            return
+            return self.check_html_w3()
         import tidy
         options = dict(output_html=0, show_warnings=1, quiet=True,
             input_encoding='utf8', output_encoding='utf8', tidy_mark=0)
@@ -802,19 +808,15 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: tidy HTML parsing caused error: %(msg)s ") %
+                _("tidy HTML parsing caused error: %(msg)s ") %
                 {"msg": err})
 
     def check_css (self):
         """Check CSS syntax of this page (which is supposed to be CSS)
         with the local cssutils module."""
-        try:
-            import cssutils
-        except ImportError:
-            log.warn(LOG_CHECK,
-                _("warning: cssutils module is not available; " \
-                 "download from http://cthedot.de/cssutils/"))
-            return
+        if not fileutil.has_module("cssutils"):
+            return self.check_css_w3()
+        import cssutils
         try:
             csslog = logging.getLogger('cssutils')
             csslog.propagate = 0
@@ -834,7 +836,7 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: cssutils parsing caused error: %(msg)s") %
+                _("cssutils parsing caused error: %(msg)s") %
                 {"msg": err})
 
     def check_w3_errors (self, xml, w3type):
@@ -874,7 +876,7 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: HTML W3C validation caused error: %(msg)s ") %
+                _("HTML W3C validation caused error: %(msg)s ") %
                 {"msg": err})
 
     def check_css_w3 (self):
@@ -909,7 +911,7 @@ class UrlBase (object):
             # errors to propagate into this library
             err = str(sys.exc_info()[1])
             log.warn(LOG_CHECK,
-                _("warning: CSS W3C validation caused error: %(msg)s ") %
+                _("CSS W3C validation caused error: %(msg)s ") %
                 {"msg": err})
 
     def scan_virus (self):
@@ -941,6 +943,8 @@ class UrlBase (object):
         # construct parser object
         handler = linkparse.LinkFinder(self.add_url)
         parser = htmlsax.parser(handler)
+        if self.charset:
+            parser.encoding = self.charset
         handler.parser = parser
         # parse
         try:
