@@ -27,7 +27,7 @@ import urlparse
 import shutil
 import _LinkChecker_configdata as configdata
 from .. import (log, LOG_CHECK, LOG_ROOT, ansicolor, lognames, clamav,
-    get_config_dir, fileutil)
+    get_config_dir, fileutil, configdict)
 from . import confparse
 from ..decorators import memoized
 
@@ -37,8 +37,8 @@ AppName = configdata.appname
 App = AppName+u" "+Version
 Author = configdata.author
 HtmlAuthor = Author.replace(u' ', u'&nbsp;')
-Copyright = u"Copyright (C) 2000-2011 "+Author
-HtmlCopyright = u"Copyright &copy; 2000-2011 "+HtmlAuthor
+Copyright = u"Copyright (C) 2000-2012 "+Author
+HtmlCopyright = u"Copyright &copy; 2000-2012 "+HtmlAuthor
 AppInfo = App+u"              "+Copyright
 HtmlAppInfo = App+u", "+HtmlCopyright
 Url = configdata.url
@@ -50,23 +50,7 @@ Freeware = AppName+u""" comes with ABSOLUTELY NO WARRANTY!
 This is free software, and you are welcome to redistribute it
 under certain conditions. Look at the file `LICENSE' within this
 distribution."""
-
-# logging configuration
-configdict = {
-    'version': 1,
-    'loggers': {
-    },
-    'root': {
-      'level': 'DEBUG',
-    },
-}
-
-# configure the application loggers
-for applog in lognames.values():
-    # propagate except for root app logger 'linkcheck'
-    propagate = (applog != LOG_ROOT)
-    configdict['loggers'][applog] = dict(level='INFO', propagate=propagate)
-
+Portable = configdata.portable
 
 def normpath (path):
     """Norm given system path with all available norm or expand functions
@@ -136,6 +120,7 @@ class Configuration (dict):
         self["wait"] = 0
         self['sendcookies'] = False
         self['storecookies'] = False
+        self['cookiefile'] = None
         self["status"] = False
         self["status_wait_seconds"] = 5
         self["fileoutput"] = []
@@ -300,7 +285,7 @@ class Configuration (dict):
         """Add given authentication data."""
         if not user or not pattern:
             log.warn(LOG_CHECK,
-            _("warning: missing user or URL pattern in authentication data."))
+            _("missing user or URL pattern in authentication data."))
             return
         entry = dict(
             user=user,
@@ -326,13 +311,9 @@ class Configuration (dict):
             self.sanitize_anchors()
         if self['logger'] is None:
             self.sanitize_logger()
-        if self['checkhtml']:
-            self.sanitize_checkhtml()
-        if self['checkcss']:
-            self.sanitize_checkcss()
         if self['scanvirus']:
             self.sanitize_scanvirus()
-        if self['storecookies']:
+        if self['storecookies'] or self['cookiefile']:
             self.sanitize_cookies()
         if self['loginurl']:
             self.sanitize_loginurl()
@@ -350,41 +331,27 @@ class Configuration (dict):
     def sanitize_logger (self):
         """Make logger configuration consistent."""
         if not self['output']:
-            log.warn(LOG_CHECK, _("warning: activating text logger output."))
+            log.warn(LOG_CHECK, _("activating text logger output."))
             self['output'] = 'text'
         self['logger'] = self.logger_new(self['output'])
 
-    def sanitize_checkhtml (self):
-        """Ensure HTML tidy is installed for checking HTML."""
-        if not fileutil.has_module("tidy"):
-            log.warn(LOG_CHECK,
-                _("warning: tidy module is not available; " \
-                 "download from http://utidylib.berlios.de/"))
-            self['checkhtml'] = False
-
-    def sanitize_checkcss (self):
-        """Ensure cssutils is installed for checking CSS."""
-        if not fileutil.has_module("cssutils"):
-            log.warn(LOG_CHECK,
-                _("warning: cssutils module is not available; " \
-                 "download from http://cthedot.de/cssutils/"))
-            self['checkcss'] = False
-
     def sanitize_scanvirus (self):
         """Ensure clamav is installed for virus checking."""
         try:
             clamav.init_clamav_conf(self['clamavconf'])
         except clamav.ClamavError:
             log.warn(LOG_CHECK,
-                _("warning: Clamav could not be initialized"))
+                _("Clamav could not be initialized"))
             self['scanvirus'] = False
 
     def sanitize_cookies (self):
         """Make cookie configuration consistent."""
         if not self['sendcookies']:
-            log.warn(LOG_CHECK, _("warning: activating sendcookies " \
-                                  "because storecookies is active."))
+            log.warn(LOG_CHECK, _("activating sendcookies."))
             self['sendcookies'] = True
+        if not self['storecookies']:
+            log.warn(LOG_CHECK, _("activating storecookies."))
+            self['storecookies'] = True
 
     def sanitize_loginurl (self):
         """Make login configuration consistent."""
@@ -392,26 +359,26 @@ class Configuration (dict):
         disable = False
         if not self["loginpasswordfield"]:
             log.warn(LOG_CHECK,
-            _("warning: no CGI password fieldname given for login URL."))
+            _("no CGI password fieldname given for login URL."))
             disable = True
         if not self["loginuserfield"]:
             log.warn(LOG_CHECK,
-            _("warning: no CGI user fieldname given for login URL."))
+            _("no CGI user fieldname given for login URL."))
             disable = True
         if self.get_user_password(url) == (None, None):
             log.warn(LOG_CHECK,
-            _("warning: no user/password authentication data found for login URL."))
+            _("no user/password authentication data found for login URL."))
             disable = True
         if not url.lower().startswith(("http:", "https:")):
-            log.warn(LOG_CHECK, _("warning: login URL is not a HTTP URL."))
+            log.warn(LOG_CHECK, _("login URL is not a HTTP URL."))
             disable = True
         urlparts = urlparse.urlsplit(url)
         if not urlparts[0] or not urlparts[1] or not urlparts[2]:
-            log.warn(LOG_CHECK, _("warning: login URL is incomplete."))
+            log.warn(LOG_CHECK, _("login URL is incomplete."))
             disable = True
         if disable:
             log.warn(LOG_CHECK,
-              _("warning: disabling login URL %(url)s.") % {"url": url})
+              _("disabling login URL %(url)s.") % {"url": url})
             self["loginurl"] = None
         elif not self['storecookies']:
             # login URL implies storing and sending cookies
@@ -434,14 +401,17 @@ class Configuration (dict):
 
 def get_user_config():
     """Find user configuration file.
+    On portable installations, an internal example configfile is returned.
     Generate user configuration file from the example configuration
     if it does not yet exist.
-    Returns path to user config file"""
+    Returns path to user config file (which might not exist do to copy
+    failures)."""
     # example config
     exampleconf = normpath(os.path.join(get_config_dir(), "linkcheckerrc"))
     # per user config settings
     userconf = normpath("~/.linkchecker/linkcheckerrc")
-    if os.path.isfile(exampleconf) and not os.path.exists(userconf):
+    if os.path.isfile(exampleconf) and not os.path.exists(userconf) and \
+       not Portable:
         # copy the system configuration to the user configuration
         try:
             userdir = os.path.dirname(userconf)
