@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2011 Bastian Kleineidam
+# Copyright (C) 2000-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -131,6 +131,12 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
             self.config["nntpserver"] = self.get(section, "nntpserver")
         if self.has_option(section, "useragent"):
             self.config["useragent"] = self.get(section, "useragent")
+        if self.has_option(section, "pause"):
+            num = self.getint(section, "pause")
+            if num < 0:
+                raise LinkCheckerError(
+                    _("invalid negative value for pause: %d\n") % num)
+            self.config["wait"] = num
         self.read_check_options(section)
 
     def read_check_options (self, section):
@@ -151,6 +157,8 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
         if self.has_option(section, "cookies"):
             self.config["sendcookies"] = self.config["storecookies"] = \
                 self.getboolean(section, "cookies")
+        if self.has_option(section, "cookiefile"):
+            self.config['cookiefile'] = self.get(section, 'cookiefile')
 
     def read_authentication_config (self):
         """Read configuration options in section "authentication"."""
