@@ -35,6 +35,7 @@ from .urlmodel import UrlItemModel
 from .urlsave import urlsave
 from .settings import Settings
 from .recentdocs import RecentDocumentModel
+from .projects import openproject, saveproject, loadproject, ProjectExt
 from .. import configuration, checker, director, add_intern_pattern, \
     strformat, fileutil, LinkCheckerError, get_link_pat
 from ..containers import enum
@@ -86,7 +87,7 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
     log_stats_signal = QtCore.pyqtSignal(object)
     error_signal = QtCore.pyqtSignal(str)
 
-    def __init__(self, parent=None, url=None):
+    def __init__(self, parent=None, url=None, project=None):
         """Initialize UI."""
         super(LinkCheckerMain, self).__init__(parent)
         self.setupUi(self)
@@ -113,10 +114,12 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.init_url(url)
         self.init_treeview()
         self.connect_widgets()
+        self.init_shortcuts()
         self.init_config()
         self.read_config()
         self.init_menu()
-        self.init_app()
+        self.init_drop()
+        self.init_app(project)
 
     def init_url (self, url):
         """Initialize URL input."""
@@ -132,7 +135,14 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         """Add menu entries for bookmark file checking."""
         self.urlinput.addMenuEntries(self.menuEdit)
 
-    def init_app (self):
+    def init_drop(self):
+        """Set and activate drag-and-drop functions."""
+        self.__class__.dragEnterEvent = self.handleDragEvent
+        self.__class__.dragMoveEvent = self.handleDragEvent
+        self.__class__.dropEvent = self.handleDropEvent
+        self.setAcceptDrops(True)
+
+    def init_app (self, project):
         """Set window size and position, GUI options and reset status."""
         data = self.settings.read_geometry()
         if data["size"] is not None:
@@ -142,8 +152,11 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.options.set_options(self.settings.read_options())
         self.status = Status.idle
         self.actionSave.setEnabled(False)
-        msg = self.config_error or _("Ready.")
-        self.set_statusmsg(msg)
+        if project:
+            loadproject(self, project)
+        else:
+            msg = self.config_error or _("Ready.")
+            self.set_statusmsg(msg)
 
     def get_qhcpath (self):
         """Helper function to search for the QHC help file in different
@@ -179,6 +192,15 @@ class LinkCheckerMain (QtGui.QMainWindow, Ui_MainWindow):
         self.options.editor.saved.connect(self.read_config)
         self.log_status_signal.connect(self.log_status)
 
+    def init_shortcuts (self):
+        """Configure application shortcuts."""
+        def selectUrl():
+            """Highlight URL input textbox."""
+            self.urlinput.setFocus()
+            self.urlinput.selectAll()
+        shortcut = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+L"), self)
+        shortcut.activated.connect(selectUrl)
+
     def init_treeview (self):
         """Set treeview model and layout."""
         self.model = UrlItemModel()
@@ -356,19 +378,24 @@ Version 2 or later.
 <a href="%(donateurl)s">donate</a>. Thanks!
 </center></qt>""") % d)
 
-    @QtCore.pyqtSlot()
-    def on_actionDonate_triggered (self):
-        """Open donation URL."""
-        webbrowser.open(configuration.DonateUrl)
-
     @QtCore.pyqtSlot()
     def on_actionDebug_triggered (self):
         """Display debug dialog."""
         self.debug.show()
 
+    @QtCore.pyqtSlot()
+    def on_actionOpen_project_triggered (self):
+        """Open project."""
+        openproject(self)
+
+    @QtCore.pyqtSlot()
+    def on_actionSave_project_triggered (self):
+        """Save project."""
+        saveproject(self, self.get_url())
+
     @QtCore.pyqtSlot()
     def on_actionSave_triggered (self):
-        """Quit application."""
+        """Save URL results."""
         urlsave(self, self.config, self.model.urls)
 
     @QtCore.pyqtSlot()
@@ -513,6 +540,7 @@ Version 2 or later.
 
     def set_statusmsg (self, msg):
         """Show given status message."""
+        self.statusBar.showMessage(msg)
         if len(msg) > 30:
             self.label_status.setToolTip(msg)
             msg = msg[:27]+u"..."
@@ -531,3 +559,22 @@ Version 2 or later.
     def internal_error (self, msg):
         """Display internal error message. Triggered by sys.excepthook()."""
         QtGui.QMessageBox.warning(self, _(u"LinkChecker internal error"), msg)
+
+    def handleDragEvent(self, event):
+        """Handle drag enter of move event."""
+        mime = event.mimeData()
+        if not mime.hasUrls():
+            return event.ignore()
+        url = mime.urls()[0]
+        if url.scheme() != 'file':
+            return event.ignore()
+        if not url.path().toLower().endsWith(ProjectExt):
+            return event.ignore()
+        event.accept()
+
+    def handleDropEvent(self, event):
+        """Handle drop event."""
+        mime = event.mimeData()
+        url = mime.urls()[0]
+        filename = unicode(url.toLocalFile())
+        loadproject(self, filename)
