@@ -27,6 +27,7 @@ import linkcheck.configuration
 import linkcheck.director
 import linkcheck.logger
 import linkcheck.i18n
+from .. import get_file
 
 # helper alias
 get_url_from = linkcheck.checker.get_url_from
@@ -98,16 +99,6 @@ class TestLogger (linkcheck.logger.Logger):
             self.diff.append(line)
 
 
-def get_file (filename=None):
-    """
-    Get file name located within 'data' directory.
-    """
-    directory = os.path.join("tests", "checker", "data")
-    if filename:
-        return unicode(os.path.join(directory, filename))
-    return unicode(directory)
-
-
 def get_file_url (filename):
     return re.sub("^([a-zA-Z]):", r"/\1|", filename.replace("\\", "/"))
 
@@ -175,6 +166,9 @@ class LinkCheckTest (unittest.TestCase):
         d = {'curdir': get_file_url(os.getcwd()),
              'datadir': get_file_url(get_file()),
             }
+        # the webserver uses the first free port number
+        if hasattr(self, 'port'):
+            d['port'] = self.port
         # all result files are encoded in utf-8
         with codecs.open(resultfile, "r", "utf-8") as f:
             return [line.rstrip(u'\r\n') % d for line in f
